# -*- coding: utf-8 -*-
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

import libear
import libscanbuild.clang as sut
import unittest
import os.path
import sys


class ClangGetVersion(unittest.TestCase):
    def test_get_version_is_not_empty(self):
        self.assertTrue(sut.get_version('clang'))

    def test_get_version_throws(self):
        with self.assertRaises(OSError):
            sut.get_version('notexists')


class ClangGetArgumentsTest(unittest.TestCase):
    def test_get_clang_arguments(self):
        with libear.TemporaryDirectory() as tmpdir:
            filename = os.path.join(tmpdir, 'test.c')
            with open(filename, 'w') as handle:
                handle.write('')

            result = sut.get_arguments(
                ['clang', '-c', filename, '-DNDEBUG', '-Dvar="this is it"'],
                tmpdir)

            self.assertTrue('NDEBUG' in result)
            self.assertTrue('var="this is it"' in result)

    def test_get_clang_arguments_fails(self):
        with self.assertRaises(Exception):
            sut.get_arguments(['clang', '-x', 'c', 'notexist.c'], '.')

    def test_get_clang_arguments_fails_badly(self):
        with self.assertRaises(OSError):
            sut.get_arguments(['notexist'], '.')


class ClangGetCheckersTest(unittest.TestCase):
    def test_get_checkers(self):
        # this test is only to see is not crashing
        result = sut.get_checkers('clang', [])
        self.assertTrue(len(result))
        # do check result types
        string_type = unicode if sys.version_info < (3,) else str
        for key, value in result.items():
            self.assertEqual(string_type, type(key))
            self.assertEqual(string_type, type(value[0]))
            self.assertEqual(bool, type(value[1]))

    def test_get_active_checkers(self):
        # this test is only to see is not crashing
        result = sut.get_active_checkers('clang', [])
        self.assertTrue(len(result))
        # do check result types
        for value in result:
            self.assertEqual(str, type(value))

    def test_is_active(self):
        test = sut.is_active(['a', 'b.b', 'c.c.c'])

        self.assertTrue(test('a'))
        self.assertTrue(test('a.b'))
        self.assertTrue(test('b.b'))
        self.assertTrue(test('b.b.c'))
        self.assertTrue(test('c.c.c.p'))

        self.assertFalse(test('ab'))
        self.assertFalse(test('ba'))
        self.assertFalse(test('bb'))
        self.assertFalse(test('c.c'))
        self.assertFalse(test('b'))
        self.assertFalse(test('d'))

    def test_parse_checkers(self):
        lines = [
            'OVERVIEW: Clang Static Analyzer Checkers List',
            '',
            'CHECKERS:',
            '  checker.one       Checker One description',
            '  checker.two',
            '                    Checker Two description']
        result = dict(sut.parse_checkers(lines))
        self.assertTrue('checker.one' in result)
        self.assertEqual('Checker One description', result.get('checker.one'))
        self.assertTrue('checker.two' in result)
        self.assertEqual('Checker Two description', result.get('checker.two'))


class ClangIsCtuCapableTest(unittest.TestCase):
    def test_ctu_not_found(self):
        is_ctu = sut.is_ctu_capable('not-found-clang-extdef-mapping')
        self.assertFalse(is_ctu)


class ClangGetTripleArchTest(unittest.TestCase):
    def test_arch_is_not_empty(self):
        arch = sut.get_triple_arch(['clang', '-E', '-'], '.')
        self.assertTrue(len(arch) > 0)



#!/usr/bin/env python

"""
Script to Summarize statistics in the scan-build output.

Statistics are enabled by passing '-internal-stats' option to scan-build
(or '-analyzer-stats' to the analyzer).
"""
from __future__ import absolute_import, division, print_function

import sys

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('Usage: ', sys.argv[0],\
                             'scan_build_output_file', file=sys.stderr)
        sys.exit(-1)

    f = open(sys.argv[1], 'r')
    Time = 0.0
    TotalTime = 0.0
    MaxTime = 0.0
    Warnings = 0
    Count = 0
    FunctionsAnalyzed = 0
    ReachableBlocks = 0
    ReachedMaxSteps = 0
    NumSteps = 0
    NumInlinedCallSites = 0
    NumBifurcatedCallSites = 0
    MaxCFGSize = 0
    for line in f:
        if ("Analyzer Total Time" in line):
            s = line.split()
            Time = Time + float(s[6])
            Count = Count + 1
            if (float(s[6]) > MaxTime):
                MaxTime = float(s[6])
        if ("warning generated." in line) or ("warnings generated" in line):
            s = line.split()
            Warnings = Warnings + int(s[0])
        if "The # of functions analysed (as top level)" in line:
            s = line.split()
            FunctionsAnalyzed = FunctionsAnalyzed + int(s[0])
        if "The % of reachable basic blocks" in line:
            s = line.split()
            ReachableBlocks = ReachableBlocks + int(s[0])
        if "The # of times we reached the max number of steps" in line:
            s = line.split()
            ReachedMaxSteps = ReachedMaxSteps + int(s[0])
        if "The maximum number of basic blocks in a function" in line:
            s = line.split()
            if MaxCFGSize < int(s[0]):
                MaxCFGSize = int(s[0])
        if "The # of steps executed" in line:
            s = line.split()
            NumSteps = NumSteps + int(s[0])
        if "The # of times we inlined a call" in line:
            s = line.split()
            NumInlinedCallSites = NumInlinedCallSites + int(s[0])
        if "The # of times we split the path due \
                to imprecise dynamic dispatch info" in line:
            s = line.split()
            NumBifurcatedCallSites = NumBifurcatedCallSites + int(s[0])
        if ")  Total" in line:
            s = line.split()
            TotalTime = TotalTime + float(s[6])

    print("TU Count %d" % (Count))
    print("Time %f" % (Time))
    print("Warnings %d" % (Warnings))
    print("Functions Analyzed %d" % (FunctionsAnalyzed))
    print("Reachable Blocks %d" % (ReachableBlocks))
    print("Reached Max Steps %d" % (ReachedMaxSteps))
    print("Number of Steps %d" % (NumSteps))
    print("Number of Inlined calls %d (bifurcated %d)" % (
        NumInlinedCallSites, NumBifurcatedCallSites))
    print("MaxTime %f" % (MaxTime))
    print("TotalTime %f" % (TotalTime))
    print("Max CFG Size %d" % (MaxCFGSize))



# -*- Python -*-

# Configuration file for the 'lit' test runner.

import os

import lit.formats

# name: The name of this test suite.
config.name = 'lld-Unit'

# suffixes: A list of file extensions to treat as test files.
config.suffixes =  []

# test_source_root: The root path where unit test binaries are located.
# test_exec_root: The root path where tests should be run.
config.test_source_root = os.path.join(config.lld_obj_root, 'unittests')
config.test_exec_root = config.test_source_root


# Tweak the PATH to include the tools dir.
path = os.path.pathsep.join((config.lld_tools_dir, config.llvm_tools_dir, config.environment['PATH']))
config.environment['PATH'] = path

path = os.path.pathsep.join((config.lld_libs_dir, config.llvm_libs_dir,
                              config.environment.get('LD_LIBRARY_PATH','')))
config.environment['LD_LIBRARY_PATH'] = path

# Propagate LLVM_SRC_ROOT into the environment.
config.environment['LLVM_SRC_ROOT'] = config.llvm_src_root

# Propagate PYTHON_EXECUTABLE into the environment
config.environment['PYTHON_EXECUTABLE'] = sys.executable


# testFormat: The test format to use to interpret tests.
config.test_format = lit.formats.GoogleTest(config.llvm_build_mode, 'Tests')



import gdb
import re

# GDB Pretty Printers for most isl objects
class IslObjectPrinter:
	"""Print an isl object"""
	def __init__ (self, val, type):
		self.val = val
		self.type = type

	def to_string (self):
		# Cast val to a void pointer to stop gdb using this pretty
		# printer for the pointer which would lead to an infinite loop.
		void_ptr = gdb.lookup_type('void').pointer()
		value = str(self.val.cast(void_ptr))
		printer = gdb.parse_and_eval("isl_printer_to_str(isl_"
					     + str(self.type)
					     + "_get_ctx(" + value + "))")
		printer = gdb.parse_and_eval("isl_printer_print_"
					     + str(self.type) + "("
					     + str(printer) + ", "
					     + value + ")")
		string = gdb.parse_and_eval("(char*)isl_printer_get_str("
					    + str(printer) + ")")
		gdb.parse_and_eval("isl_printer_free(" + str(printer) + ")")
		return string

	def display_hint (self):
		return 'string'

class IslIntPrinter:
	"""Print an isl_int """
	def __init__ (self, val):
		self.val = val

	def to_string (self):
		# Cast val to a void pointer to stop gdb using this pretty
		# printer for the pointer which would lead to an infinite loop.
		void_ptr = gdb.lookup_type('void').pointer()
		value = str(self.val.cast(void_ptr))

		context = gdb.parse_and_eval("isl_ctx_alloc()")
		printer = gdb.parse_and_eval("isl_printer_to_str("
					     + str(context) + ")")
		printer = gdb.parse_and_eval("isl_printer_print_isl_int("
					     + str(printer) + ", "
					     + value + ")")
		string = gdb.parse_and_eval("(char*)isl_printer_get_str("
					    + str(printer) + ")")
		gdb.parse_and_eval("isl_printer_free(" + str(printer) + ")")
		gdb.parse_and_eval("isl_ctx_free(" + str(context) + ")")
		return string

	def display_hint (self):
		return 'string'

class IslPrintCommand (gdb.Command):
	"""Print an isl value."""
	def __init__ (self):
		super (IslPrintCommand, self).__init__ ("islprint",
							gdb.COMMAND_OBSCURE)
	def invoke (self, arg, from_tty):
		arg = gdb.parse_and_eval(arg);
		printer = str_lookup_function(arg)

		if printer == None:
			print("No isl printer for this type")
			return

		print(printer.to_string())

IslPrintCommand()

def str_lookup_function (val):
	if val.type.code != gdb.TYPE_CODE_PTR:
		if str(val.type) == "isl_int":
			return IslIntPrinter(val)
		else:
			return None

	lookup_tag = val.type.target()
	regex = re.compile ("^isl_(.*)$")

	if lookup_tag == None:
		return None

	m = regex.match (str(lookup_tag))

	if m:
		# Those types of printers defined in isl.
		if m.group(1) in ["basic_set", "set", "union_set", "basic_map",
				  "map", "union_map", "qpolynomial",
				  "pw_qpolynomial", "pw_qpolynomial_fold",
				  "union_pw_qpolynomial",
				  "union_pw_qpolynomial_fold"]:
			return IslObjectPrinter(val, m.group(1))
	return None

# Do not register the pretty printer.
# gdb.current_objfile().pretty_printers.append(str_lookup_function)



import os
from subprocess import check_call
import sys


Verbose = 1

def which(command, paths=None):
    """which(command, [paths]) - Look up the given command in the paths string
    (or the PATH environment variable, if unspecified)."""

    if paths is None:
        paths = os.environ.get('PATH', '')

    # Check for absolute match first.
    if os.path.exists(command):
        return command

    # Would be nice if Python had a lib function for this.
    if not paths:
        paths = os.defpath

    # Get suffixes to search.
    # On Cygwin, 'PATHEXT' may exist but it should not be used.
    if os.pathsep == ';':
        pathext = os.environ.get('PATHEXT', '').split(';')
    else:
        pathext = ['']

    # Search the paths...
    for path in paths.split(os.pathsep):
        for ext in pathext:
            p = os.path.join(path, command + ext)
            if os.path.exists(p):
                return p

    return None


def hasNoExtension(FileName):
    (Root, Ext) = os.path.splitext(FileName)
    return (Ext == "")


def isValidSingleInputFile(FileName):
    (Root, Ext) = os.path.splitext(FileName)
    return Ext in (".i", ".ii", ".c", ".cpp", ".m", "")


def runScript(ScriptPath, PBuildLogFile, Cwd, Stdout=sys.stdout,
              Stderr=sys.stderr):
    """
    Run the provided script if it exists.
    """
    if os.path.exists(ScriptPath):
        try:
            if Verbose == 1:
                Stdout.write("  Executing: %s\n" % (ScriptPath,))
            check_call("chmod +x '%s'" % ScriptPath, cwd=Cwd,
                       stderr=PBuildLogFile,
                       stdout=PBuildLogFile,
                       shell=True)
            check_call("'%s'" % ScriptPath, cwd=Cwd,
                       stderr=PBuildLogFile,
                       stdout=PBuildLogFile,
                       shell=True)
        except:
            Stderr.write("Error: Running %s failed. See %s for details.\n" % (
                         ScriptPath, PBuildLogFile.name))
            sys.exit(-1)


def isCommentCSVLine(Entries):
    """
    Treat CSV lines starting with a '#' as a comment.
    """
    return len(Entries) > 0 and Entries[0].startswith("#")
